# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/tmp/tmpeh68xiaa'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.setWindowModality(QtCore.Qt.NonModal)
        MainWindow.resize(800, 696)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/images/DXF2GCODE-001.ico"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        MainWindow.setWindowIcon(icon)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setContentsMargins(-1, -1, -1, 2)
        self.verticalLayout.setSpacing(6)
        self.verticalLayout.setObjectName("verticalLayout")
        self.splitter = QtWidgets.QSplitter(self.centralwidget)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setChildrenCollapsible(False)
        self.splitter.setObjectName("splitter")
        self.mytabWidget = QtWidgets.QTabWidget(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(2)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.mytabWidget.sizePolicy().hasHeightForWidth())
        self.mytabWidget.setSizePolicy(sizePolicy)
        self.mytabWidget.setMinimumSize(QtCore.QSize(200, 0))
        self.mytabWidget.setAutoFillBackground(False)
        self.mytabWidget.setObjectName("mytabWidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.tab)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setSpacing(1)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.blocksCollapsePushButton = QtWidgets.QPushButton(self.tab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.blocksCollapsePushButton.sizePolicy().hasHeightForWidth())
        self.blocksCollapsePushButton.setSizePolicy(sizePolicy)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/images/collapse-all.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.blocksCollapsePushButton.setIcon(icon1)
        self.blocksCollapsePushButton.setIconSize(QtCore.QSize(24, 24))
        self.blocksCollapsePushButton.setObjectName("blocksCollapsePushButton")
        self.horizontalLayout_5.addWidget(self.blocksCollapsePushButton)
        self.blocksExpandPushButton = QtWidgets.QPushButton(self.tab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.blocksExpandPushButton.sizePolicy().hasHeightForWidth())
        self.blocksExpandPushButton.setSizePolicy(sizePolicy)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/images/expand-all.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.blocksExpandPushButton.setIcon(icon2)
        self.blocksExpandPushButton.setIconSize(QtCore.QSize(24, 24))
        self.blocksExpandPushButton.setObjectName("blocksExpandPushButton")
        self.horizontalLayout_5.addWidget(self.blocksExpandPushButton)
        spacerItem = QtWidgets.QSpacerItem(13, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem)
        self.verticalLayout_3.addLayout(self.horizontalLayout_5)
        self.entitiesTreeView = TreeView(self.tab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.entitiesTreeView.sizePolicy().hasHeightForWidth())
        self.entitiesTreeView.setSizePolicy(sizePolicy)
        self.entitiesTreeView.setObjectName("entitiesTreeView")
        self.verticalLayout_3.addWidget(self.entitiesTreeView)
        self.mytabWidget.addTab(self.tab, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.tab_2)
        self.verticalLayout_5.setContentsMargins(5, 5, 5, 5)
        self.verticalLayout_5.setSpacing(1)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setSpacing(1)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.layersCollapsePushButton = QtWidgets.QPushButton(self.tab_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.layersCollapsePushButton.sizePolicy().hasHeightForWidth())
        self.layersCollapsePushButton.setSizePolicy(sizePolicy)
        self.layersCollapsePushButton.setIcon(icon1)
        self.layersCollapsePushButton.setIconSize(QtCore.QSize(24, 24))
        self.layersCollapsePushButton.setObjectName("layersCollapsePushButton")
        self.horizontalLayout_4.addWidget(self.layersCollapsePushButton)
        self.layersExpandPushButton = QtWidgets.QPushButton(self.tab_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.layersExpandPushButton.sizePolicy().hasHeightForWidth())
        self.layersExpandPushButton.setSizePolicy(sizePolicy)
        self.layersExpandPushButton.setIcon(icon2)
        self.layersExpandPushButton.setIconSize(QtCore.QSize(24, 24))
        self.layersExpandPushButton.setObjectName("layersExpandPushButton")
        self.horizontalLayout_4.addWidget(self.layersExpandPushButton)
        spacerItem1 = QtWidgets.QSpacerItem(13, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem1)
        self.layersGoUpPushButton = QtWidgets.QPushButton(self.tab_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.layersGoUpPushButton.sizePolicy().hasHeightForWidth())
        self.layersGoUpPushButton.setSizePolicy(sizePolicy)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/images/go-up.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.layersGoUpPushButton.setIcon(icon3)
        self.layersGoUpPushButton.setIconSize(QtCore.QSize(24, 24))
        self.layersGoUpPushButton.setObjectName("layersGoUpPushButton")
        self.horizontalLayout_4.addWidget(self.layersGoUpPushButton)
        self.layersGoDownPushButton = QtWidgets.QPushButton(self.tab_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.layersGoDownPushButton.sizePolicy().hasHeightForWidth())
        self.layersGoDownPushButton.setSizePolicy(sizePolicy)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/images/go-down.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.layersGoDownPushButton.setIcon(icon4)
        self.layersGoDownPushButton.setIconSize(QtCore.QSize(24, 24))
        self.layersGoDownPushButton.setObjectName("layersGoDownPushButton")
        self.horizontalLayout_4.addWidget(self.layersGoDownPushButton)
        self.verticalLayout_5.addLayout(self.horizontalLayout_4)
        self.layersShapesTreeView = TreeView(self.tab_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.layersShapesTreeView.sizePolicy().hasHeightForWidth())
        self.layersShapesTreeView.setSizePolicy(sizePolicy)
        self.layersShapesTreeView.setObjectName("layersShapesTreeView")
        self.verticalLayout_5.addWidget(self.layersShapesTreeView)
        self.millSettingsFrame = QtWidgets.QFrame(self.tab_2)
        self.millSettingsFrame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.millSettingsFrame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.millSettingsFrame.setLineWidth(0)
        self.millSettingsFrame.setObjectName("millSettingsFrame")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.millSettingsFrame)
        self.verticalLayout_4.setContentsMargins(2, 2, 2, 2)
        self.verticalLayout_4.setSpacing(0)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setSpacing(2)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.toolDiameterComboBox = QtWidgets.QComboBox(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.toolDiameterComboBox.sizePolicy().hasHeightForWidth())
        self.toolDiameterComboBox.setSizePolicy(sizePolicy)
        self.toolDiameterComboBox.setMaxVisibleItems(20)
        self.toolDiameterComboBox.setObjectName("toolDiameterComboBox")
        self.horizontalLayout_3.addWidget(self.toolDiameterComboBox)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_11 = QtWidgets.QLabel(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_11.sizePolicy().hasHeightForWidth())
        self.label_11.setSizePolicy(sizePolicy)
        self.label_11.setText("⌀")
        self.label_11.setTextFormat(QtCore.Qt.PlainText)
        self.label_11.setObjectName("label_11")
        self.horizontalLayout.addWidget(self.label_11)
        self.toolDiameterLabel = QtWidgets.QLabel(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.toolDiameterLabel.sizePolicy().hasHeightForWidth())
        self.toolDiameterLabel.setSizePolicy(sizePolicy)
        self.toolDiameterLabel.setText("[mm]")
        self.toolDiameterLabel.setObjectName("toolDiameterLabel")
        self.horizontalLayout.addWidget(self.toolDiameterLabel)
        self.label_12 = QtWidgets.QLabel(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_12.sizePolicy().hasHeightForWidth())
        self.label_12.setSizePolicy(sizePolicy)
        self.label_12.setObjectName("label_12")
        self.horizontalLayout.addWidget(self.label_12)
        self.toolSpeedLabel = QtWidgets.QLabel(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.toolSpeedLabel.sizePolicy().hasHeightForWidth())
        self.toolSpeedLabel.setSizePolicy(sizePolicy)
        self.toolSpeedLabel.setText("[rpm]")
        self.toolSpeedLabel.setObjectName("toolSpeedLabel")
        self.horizontalLayout.addWidget(self.toolSpeedLabel)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_13 = QtWidgets.QLabel(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_13.sizePolicy().hasHeightForWidth())
        self.label_13.setSizePolicy(sizePolicy)
        self.label_13.setObjectName("label_13")
        self.horizontalLayout_2.addWidget(self.label_13)
        self.startRadiusLabel = QtWidgets.QLabel(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.startRadiusLabel.sizePolicy().hasHeightForWidth())
        self.startRadiusLabel.setSizePolicy(sizePolicy)
        self.startRadiusLabel.setText("[mm]")
        self.startRadiusLabel.setObjectName("startRadiusLabel")
        self.horizontalLayout_2.addWidget(self.startRadiusLabel)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3.addLayout(self.verticalLayout_2)
        self.verticalLayout_4.addLayout(self.horizontalLayout_3)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setHorizontalSpacing(2)
        self.gridLayout.setVerticalSpacing(0)
        self.gridLayout.setObjectName("gridLayout")
        self.zInitialMillDepthLineEdit = QtWidgets.QLineEdit(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.zInitialMillDepthLineEdit.sizePolicy().hasHeightForWidth())
        self.zInitialMillDepthLineEdit.setSizePolicy(sizePolicy)
        self.zInitialMillDepthLineEdit.setObjectName("zInitialMillDepthLineEdit")
        self.gridLayout.addWidget(self.zInitialMillDepthLineEdit, 4, 1, 1, 1)
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_13.setContentsMargins(0, -1, 0, -1)
        self.horizontalLayout_13.setSpacing(2)
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.label_7 = QtWidgets.QLabel(self.millSettingsFrame)
        self.label_7.setWordWrap(True)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_13.addWidget(self.label_7)
        self.unitLabel_8 = QtWidgets.QLabel(self.millSettingsFrame)
        self.unitLabel_8.setText("[mm/min]")
        self.unitLabel_8.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.unitLabel_8.setObjectName("unitLabel_8")
        self.horizontalLayout_13.addWidget(self.unitLabel_8)
        self.gridLayout.addLayout(self.horizontalLayout_13, 7, 0, 1, 1)
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setContentsMargins(0, -1, 0, -1)
        self.horizontalLayout_12.setSpacing(2)
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.label_8 = QtWidgets.QLabel(self.millSettingsFrame)
        self.label_8.setObjectName("label_8")
        self.horizontalLayout_12.addWidget(self.label_8)
        self.unitLabel_7 = QtWidgets.QLabel(self.millSettingsFrame)
        self.unitLabel_7.setText("[mm]")
        self.unitLabel_7.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.unitLabel_7.setObjectName("unitLabel_7")
        self.horizontalLayout_12.addWidget(self.unitLabel_7)
        self.gridLayout.addLayout(self.horizontalLayout_12, 6, 0, 1, 1)
        self.zInfeedDepthLineEdit = QtWidgets.QLineEdit(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.zInfeedDepthLineEdit.sizePolicy().hasHeightForWidth())
        self.zInfeedDepthLineEdit.setSizePolicy(sizePolicy)
        self.zInfeedDepthLineEdit.setObjectName("zInfeedDepthLineEdit")
        self.gridLayout.addWidget(self.zInfeedDepthLineEdit, 5, 1, 1, 1)
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setContentsMargins(0, -1, 0, -1)
        self.horizontalLayout_11.setSpacing(2)
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.label_9 = QtWidgets.QLabel(self.millSettingsFrame)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_11.addWidget(self.label_9)
        self.unitLabel_6 = QtWidgets.QLabel(self.millSettingsFrame)
        self.unitLabel_6.setText("[mm]")
        self.unitLabel_6.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.unitLabel_6.setObjectName("unitLabel_6")
        self.horizontalLayout_11.addWidget(self.unitLabel_6)
        self.gridLayout.addLayout(self.horizontalLayout_11, 5, 0, 1, 1)
        self.zSafetyMarginLineEdit = QtWidgets.QLineEdit(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.zSafetyMarginLineEdit.sizePolicy().hasHeightForWidth())
        self.zSafetyMarginLineEdit.setSizePolicy(sizePolicy)
        self.zSafetyMarginLineEdit.setObjectName("zSafetyMarginLineEdit")
        self.gridLayout.addWidget(self.zSafetyMarginLineEdit, 3, 1, 1, 1)
        self.zFinalMillDepthLineEdit = QtWidgets.QLineEdit(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.zFinalMillDepthLineEdit.sizePolicy().hasHeightForWidth())
        self.zFinalMillDepthLineEdit.setSizePolicy(sizePolicy)
        self.zFinalMillDepthLineEdit.setObjectName("zFinalMillDepthLineEdit")
        self.gridLayout.addWidget(self.zFinalMillDepthLineEdit, 6, 1, 1, 1)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setContentsMargins(0, -1, 0, -1)
        self.horizontalLayout_9.setSpacing(2)
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.label_5 = QtWidgets.QLabel(self.millSettingsFrame)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_9.addWidget(self.label_5)
        self.unitLabel_4 = QtWidgets.QLabel(self.millSettingsFrame)
        self.unitLabel_4.setText("[mm]")
        self.unitLabel_4.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.unitLabel_4.setObjectName("unitLabel_4")
        self.horizontalLayout_9.addWidget(self.unitLabel_4)
        self.gridLayout.addLayout(self.horizontalLayout_9, 3, 0, 1, 1)
        self.zRetractionArealLineEdit = QtWidgets.QLineEdit(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.zRetractionArealLineEdit.sizePolicy().hasHeightForWidth())
        self.zRetractionArealLineEdit.setSizePolicy(sizePolicy)
        self.zRetractionArealLineEdit.setObjectName("zRetractionArealLineEdit")
        self.gridLayout.addWidget(self.zRetractionArealLineEdit, 2, 1, 1, 1)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setContentsMargins(0, -1, 0, -1)
        self.horizontalLayout_8.setSpacing(2)
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.label_6 = QtWidgets.QLabel(self.millSettingsFrame)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_8.addWidget(self.label_6)
        self.unitLabel_3 = QtWidgets.QLabel(self.millSettingsFrame)
        self.unitLabel_3.setText("[mm]")
        self.unitLabel_3.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.unitLabel_3.setObjectName("unitLabel_3")
        self.horizontalLayout_8.addWidget(self.unitLabel_3)
        self.gridLayout.addLayout(self.horizontalLayout_8, 2, 0, 1, 1)
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setContentsMargins(0, -1, 0, -1)
        self.horizontalLayout_10.setSpacing(2)
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.label_14 = QtWidgets.QLabel(self.millSettingsFrame)
        self.label_14.setObjectName("label_14")
        self.horizontalLayout_10.addWidget(self.label_14)
        self.unitLabel_5 = QtWidgets.QLabel(self.millSettingsFrame)
        self.unitLabel_5.setText("[mm]")
        self.unitLabel_5.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.unitLabel_5.setObjectName("unitLabel_5")
        self.horizontalLayout_10.addWidget(self.unitLabel_5)
        self.gridLayout.addLayout(self.horizontalLayout_10, 4, 0, 1, 1)
        self.g1FeedZLineEdit = QtWidgets.QLineEdit(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.g1FeedZLineEdit.sizePolicy().hasHeightForWidth())
        self.g1FeedZLineEdit.setSizePolicy(sizePolicy)
        self.g1FeedZLineEdit.setObjectName("g1FeedZLineEdit")
        self.gridLayout.addWidget(self.g1FeedZLineEdit, 8, 1, 1, 1)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setContentsMargins(0, -1, 0, -1)
        self.horizontalLayout_6.setSpacing(2)
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.startAtXLabel = QtWidgets.QLabel(self.millSettingsFrame)
        self.startAtXLabel.setObjectName("startAtXLabel")
        self.horizontalLayout_6.addWidget(self.startAtXLabel)
        self.unitLabel_1 = QtWidgets.QLabel(self.millSettingsFrame)
        self.unitLabel_1.setText("[mm]")
        self.unitLabel_1.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.unitLabel_1.setObjectName("unitLabel_1")
        self.horizontalLayout_6.addWidget(self.unitLabel_1)
        self.gridLayout.addLayout(self.horizontalLayout_6, 0, 0, 1, 1)
        self.startAtYLineEdit = QtWidgets.QLineEdit(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.startAtYLineEdit.sizePolicy().hasHeightForWidth())
        self.startAtYLineEdit.setSizePolicy(sizePolicy)
        self.startAtYLineEdit.setObjectName("startAtYLineEdit")
        self.gridLayout.addWidget(self.startAtYLineEdit, 1, 1, 1, 1)
        self.g1FeedXYLineEdit = QtWidgets.QLineEdit(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.g1FeedXYLineEdit.sizePolicy().hasHeightForWidth())
        self.g1FeedXYLineEdit.setSizePolicy(sizePolicy)
        self.g1FeedXYLineEdit.setObjectName("g1FeedXYLineEdit")
        self.gridLayout.addWidget(self.g1FeedXYLineEdit, 7, 1, 1, 1)
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_14.setContentsMargins(0, -1, 0, -1)
        self.horizontalLayout_14.setSpacing(2)
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        self.label_10 = QtWidgets.QLabel(self.millSettingsFrame)
        self.label_10.setWordWrap(True)
        self.label_10.setObjectName("label_10")
        self.horizontalLayout_14.addWidget(self.label_10)
        self.unitLabel_9 = QtWidgets.QLabel(self.millSettingsFrame)
        self.unitLabel_9.setText("[mm/min]")
        self.unitLabel_9.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.unitLabel_9.setObjectName("unitLabel_9")
        self.horizontalLayout_14.addWidget(self.unitLabel_9)
        self.gridLayout.addLayout(self.horizontalLayout_14, 8, 0, 1, 1)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setContentsMargins(0, -1, 0, -1)
        self.horizontalLayout_7.setSpacing(2)
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.startAtYLabel = QtWidgets.QLabel(self.millSettingsFrame)
        self.startAtYLabel.setObjectName("startAtYLabel")
        self.horizontalLayout_7.addWidget(self.startAtYLabel)
        self.unitLabel_2 = QtWidgets.QLabel(self.millSettingsFrame)
        self.unitLabel_2.setText("[mm]")
        self.unitLabel_2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.unitLabel_2.setObjectName("unitLabel_2")
        self.horizontalLayout_7.addWidget(self.unitLabel_2)
        self.gridLayout.addLayout(self.horizontalLayout_7, 1, 0, 1, 1)
        self.startAtXLineEdit = QtWidgets.QLineEdit(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.startAtXLineEdit.sizePolicy().hasHeightForWidth())
        self.startAtXLineEdit.setSizePolicy(sizePolicy)
        self.startAtXLineEdit.setObjectName("startAtXLineEdit")
        self.gridLayout.addWidget(self.startAtXLineEdit, 0, 1, 1, 1)
        self.verticalLayout_4.addLayout(self.gridLayout)
        self.verticalLayout_5.addWidget(self.millSettingsFrame)
        self.mytabWidget.addTab(self.tab_2, "")
        self.canvas = Canvas(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(7)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.canvas.sizePolicy().hasHeightForWidth())
        self.canvas.setSizePolicy(sizePolicy)
        self.canvas.setMinimumSize(QtCore.QSize(200, 200))
        self.canvas.setContextMenuPolicy(QtCore.Qt.DefaultContextMenu)
        self.canvas.setObjectName("canvas")
        self.verticalLayout.addWidget(self.splitter)
        self.messageBox = MessageBox(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.messageBox.sizePolicy().hasHeightForWidth())
        self.messageBox.setSizePolicy(sizePolicy)
        self.messageBox.setMaximumSize(QtCore.QSize(16777215, 100))
        self.messageBox.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.messageBox.setObjectName("messageBox")
        self.verticalLayout.addWidget(self.messageBox)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 800, 24))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuExport = QtWidgets.QMenu(self.menubar)
        self.menuExport.setEnabled(True)
        self.menuExport.setObjectName("menuExport")
        self.menuView = QtWidgets.QMenu(self.menubar)
        self.menuView.setObjectName("menuView")
        self.menuOptions = QtWidgets.QMenu(self.menubar)
        self.menuOptions.setObjectName("menuOptions")
        self.menuMachine_Type = QtWidgets.QMenu(self.menuOptions)
        self.menuMachine_Type.setObjectName("menuMachine_Type")
        self.menu_Measurement_units = QtWidgets.QMenu(self.menuOptions)
        self.menu_Measurement_units.setEnabled(False)
        self.menu_Measurement_units.setToolTip("")
        self.menu_Measurement_units.setObjectName("menu_Measurement_units")
        self.menuHelp = QtWidgets.QMenu(self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionOpen = QtWidgets.QAction(MainWindow)
        self.actionOpen.setObjectName("actionOpen")
        self.actionClose = QtWidgets.QAction(MainWindow)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/images/delete.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionClose.setIcon(icon5)
        self.actionClose.setObjectName("actionClose")
        self.actionShowPathDirections = QtWidgets.QAction(MainWindow)
        self.actionShowPathDirections.setCheckable(True)
        self.actionShowPathDirections.setChecked(False)
        self.actionShowPathDirections.setEnabled(False)
        self.actionShowPathDirections.setObjectName("actionShowPathDirections")
        self.actionShowDisabledPaths = QtWidgets.QAction(MainWindow)
        self.actionShowDisabledPaths.setCheckable(True)
        self.actionShowDisabledPaths.setChecked(False)
        self.actionShowDisabledPaths.setEnabled(False)
        self.actionShowDisabledPaths.setObjectName("actionShowDisabledPaths")
        self.actionAutoscale = QtWidgets.QAction(MainWindow)
        self.actionAutoscale.setEnabled(False)
        self.actionAutoscale.setObjectName("actionAutoscale")
        self.actionDeleteG0Paths = QtWidgets.QAction(MainWindow)
        self.actionDeleteG0Paths.setEnabled(False)
        self.actionDeleteG0Paths.setObjectName("actionDeleteG0Paths")
        self.actionConfiguration = QtWidgets.QAction(MainWindow)
        self.actionConfiguration.setObjectName("actionConfiguration")
        self.actionConfigurationPostprocessor = QtWidgets.QAction(MainWindow)
        self.actionConfigurationPostprocessor.setObjectName("actionConfigurationPostprocessor")
        self.actionTolerances = QtWidgets.QAction(MainWindow)
        self.actionTolerances.setObjectName("actionTolerances")
        self.actionScaleAll = QtWidgets.QAction(MainWindow)
        self.actionScaleAll.setEnabled(False)
        self.actionScaleAll.setObjectName("actionScaleAll")
        self.actionRotateAll = QtWidgets.QAction(MainWindow)
        self.actionRotateAll.setEnabled(False)
        self.actionRotateAll.setObjectName("actionRotateAll")
        self.actionMoveWorkpieceZero = QtWidgets.QAction(MainWindow)
        self.actionMoveWorkpieceZero.setEnabled(False)
        self.actionMoveWorkpieceZero.setObjectName("actionMoveWorkpieceZero")
        self.actionAbout = QtWidgets.QAction(MainWindow)
        self.actionAbout.setObjectName("actionAbout")
        self.actionOptimizePaths = QtWidgets.QAction(MainWindow)
        self.actionOptimizePaths.setEnabled(False)
        self.actionOptimizePaths.setObjectName("actionOptimizePaths")
        self.actionExportShapes = QtWidgets.QAction(MainWindow)
        self.actionExportShapes.setEnabled(False)
        self.actionExportShapes.setObjectName("actionExportShapes")
        self.actionOptimizeAndExportShapes = QtWidgets.QAction(MainWindow)
        self.actionOptimizeAndExportShapes.setEnabled(False)
        self.actionOptimizeAndExportShapes.setObjectName("actionOptimizeAndExportShapes")
        self.actionLiveUpdateExportRoute = QtWidgets.QAction(MainWindow)
        self.actionLiveUpdateExportRoute.setCheckable(True)
        self.actionLiveUpdateExportRoute.setEnabled(False)
        self.actionLiveUpdateExportRoute.setObjectName("actionLiveUpdateExportRoute")
        self.actionReload = QtWidgets.QAction(MainWindow)
        self.actionReload.setEnabled(False)
        self.actionReload.setObjectName("actionReload")
        self.actionSplitLineSegments = QtWidgets.QAction(MainWindow)
        self.actionSplitLineSegments.setCheckable(True)
        self.actionSplitLineSegments.setObjectName("actionSplitLineSegments")
        self.actionAutomaticCutterCompensation = QtWidgets.QAction(MainWindow)
        self.actionAutomaticCutterCompensation.setCheckable(True)
        self.actionAutomaticCutterCompensation.setEnabled(False)
        self.actionAutomaticCutterCompensation.setObjectName("actionAutomaticCutterCompensation")
        self.actionMilling = QtWidgets.QAction(MainWindow)
        self.actionMilling.setCheckable(True)
        self.actionMilling.setObjectName("actionMilling")
        self.actionDragKnife = QtWidgets.QAction(MainWindow)
        self.actionDragKnife.setCheckable(True)
        self.actionDragKnife.setObjectName("actionDragKnife")
        self.actionLathe = QtWidgets.QAction(MainWindow)
        self.actionLathe.setCheckable(True)
        self.actionLathe.setObjectName("actionLathe")
        self.actionTopView = QtWidgets.QAction(MainWindow)
        self.actionTopView.setEnabled(False)
        self.actionTopView.setObjectName("actionTopView")
        self.actionIsometricView = QtWidgets.QAction(MainWindow)
        self.actionIsometricView.setEnabled(False)
        self.actionIsometricView.setObjectName("actionIsometricView")
        self.actionSaveProjectAs = QtWidgets.QAction(MainWindow)
        self.actionSaveProjectAs.setEnabled(False)
        self.actionSaveProjectAs.setObjectName("actionSaveProjectAs")
        self.actionMillimeters = QtWidgets.QAction(MainWindow)
        self.actionMillimeters.setCheckable(True)
        self.actionMillimeters.setEnabled(True)
        self.actionMillimeters.setObjectName("actionMillimeters")
        self.actionInches = QtWidgets.QAction(MainWindow)
        self.actionInches.setCheckable(True)
        self.actionInches.setObjectName("actionInches")
        self.menuFile.addAction(self.actionOpen)
        self.menuFile.addAction(self.actionReload)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionSaveProjectAs)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionClose)
        self.menuExport.addAction(self.actionOptimizePaths)
        self.menuExport.addAction(self.actionExportShapes)
        self.menuExport.addAction(self.actionOptimizeAndExportShapes)
        self.menuView.addAction(self.actionShowPathDirections)
        self.menuView.addAction(self.actionShowDisabledPaths)
        self.menuView.addSeparator()
        self.menuView.addAction(self.actionLiveUpdateExportRoute)
        self.menuView.addAction(self.actionDeleteG0Paths)
        self.menuView.addSeparator()
        self.menuView.addAction(self.actionAutoscale)
        self.menuView.addAction(self.actionTopView)
        self.menuView.addAction(self.actionIsometricView)
        self.menuMachine_Type.addAction(self.actionMilling)
        self.menuMachine_Type.addAction(self.actionLathe)
        self.menuMachine_Type.addAction(self.actionDragKnife)
        self.menu_Measurement_units.addAction(self.actionMillimeters)
        self.menu_Measurement_units.addAction(self.actionInches)
        self.menuOptions.addAction(self.actionConfiguration)
        self.menuOptions.addAction(self.actionConfigurationPostprocessor)
        self.menuOptions.addSeparator()
        self.menuOptions.addAction(self.actionTolerances)
        self.menuOptions.addSeparator()
        self.menuOptions.addAction(self.actionScaleAll)
        self.menuOptions.addAction(self.actionRotateAll)
        self.menuOptions.addSeparator()
        self.menuOptions.addAction(self.actionMoveWorkpieceZero)
        self.menuOptions.addSeparator()
        self.menuOptions.addAction(self.actionSplitLineSegments)
        self.menuOptions.addAction(self.actionAutomaticCutterCompensation)
        self.menuOptions.addAction(self.menuMachine_Type.menuAction())
        self.menuOptions.addAction(self.menu_Measurement_units.menuAction())
        self.menuHelp.addAction(self.actionAbout)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuExport.menuAction())
        self.menubar.addAction(self.menuView.menuAction())
        self.menubar.addAction(self.menuOptions.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(MainWindow)
        self.mytabWidget.setCurrentIndex(1)
        self.layersCollapsePushButton.clicked.connect(self.layersShapesTreeView.collapseAll) # type: ignore
        self.layersExpandPushButton.clicked.connect(self.layersShapesTreeView.expandAll) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(MainWindow)
        MainWindow.setTabOrder(self.mytabWidget, self.entitiesTreeView)
        MainWindow.setTabOrder(self.entitiesTreeView, self.blocksCollapsePushButton)
        MainWindow.setTabOrder(self.blocksCollapsePushButton, self.blocksExpandPushButton)
        MainWindow.setTabOrder(self.blocksExpandPushButton, self.canvas)
        MainWindow.setTabOrder(self.canvas, self.messageBox)
        MainWindow.setTabOrder(self.messageBox, self.layersShapesTreeView)
        MainWindow.setTabOrder(self.layersShapesTreeView, self.layersCollapsePushButton)
        MainWindow.setTabOrder(self.layersCollapsePushButton, self.layersExpandPushButton)
        MainWindow.setTabOrder(self.layersExpandPushButton, self.layersGoUpPushButton)
        MainWindow.setTabOrder(self.layersGoUpPushButton, self.layersGoDownPushButton)
        MainWindow.setTabOrder(self.layersGoDownPushButton, self.toolDiameterComboBox)
        MainWindow.setTabOrder(self.toolDiameterComboBox, self.startAtXLineEdit)
        MainWindow.setTabOrder(self.startAtXLineEdit, self.startAtYLineEdit)
        MainWindow.setTabOrder(self.startAtYLineEdit, self.zRetractionArealLineEdit)
        MainWindow.setTabOrder(self.zRetractionArealLineEdit, self.zSafetyMarginLineEdit)
        MainWindow.setTabOrder(self.zSafetyMarginLineEdit, self.zInitialMillDepthLineEdit)
        MainWindow.setTabOrder(self.zInitialMillDepthLineEdit, self.zInfeedDepthLineEdit)
        MainWindow.setTabOrder(self.zInfeedDepthLineEdit, self.zFinalMillDepthLineEdit)
        MainWindow.setTabOrder(self.zFinalMillDepthLineEdit, self.g1FeedXYLineEdit)
        MainWindow.setTabOrder(self.g1FeedXYLineEdit, self.g1FeedZLineEdit)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "DXF2GCODE"))
        self.blocksCollapsePushButton.setToolTip(_translate("MainWindow", "Collapse all items"))
        self.blocksExpandPushButton.setToolTip(_translate("MainWindow", "Expand all items"))
        self.mytabWidget.setTabText(self.mytabWidget.indexOf(self.tab), _translate("MainWindow", "Entities"))
        self.layersCollapsePushButton.setToolTip(_translate("MainWindow", "Collapse all items"))
        self.layersExpandPushButton.setToolTip(_translate("MainWindow", "Expand all items"))
        self.layersGoUpPushButton.setToolTip(_translate("MainWindow", "Move-up the selected shape/layer"))
        self.layersGoDownPushButton.setToolTip(_translate("MainWindow", "Move-down the selected shape/layer"))
        self.label_12.setText(_translate("MainWindow", "/ speed "))
        self.label_13.setText(_translate("MainWindow", "start rad. (comp) "))
        self.zInitialMillDepthLineEdit.setToolTip(_translate("MainWindow", "Milling will start at Z = \"Workpiece top Z\" - \"Z infeed depth\""))
        self.label_7.setText(_translate("MainWindow", "Feed rate XY"))
        self.label_8.setText(_translate("MainWindow", "Z Final mill depth"))
        self.label_9.setText(_translate("MainWindow", "Z Infeed depth"))
        self.label_5.setText(_translate("MainWindow", "Z Safety margin"))
        self.label_6.setText(_translate("MainWindow", "Z Retraction area"))
        self.label_14.setText(_translate("MainWindow", "Z Workpiece top"))
        self.startAtXLabel.setText(_translate("MainWindow", "Start X"))
        self.label_10.setText(_translate("MainWindow", "Feed rate Z"))
        self.startAtYLabel.setText(_translate("MainWindow", "Start Y"))
        self.mytabWidget.setTabText(self.mytabWidget.indexOf(self.tab_2), _translate("MainWindow", "Layers"))
        self.menuFile.setTitle(_translate("MainWindow", "&File"))
        self.menuExport.setStatusTip(_translate("MainWindow", "Export the current project to G-Code"))
        self.menuExport.setTitle(_translate("MainWindow", "&Export"))
        self.menuView.setTitle(_translate("MainWindow", "&View"))
        self.menuOptions.setTitle(_translate("MainWindow", "&Options"))
        self.menuMachine_Type.setTitle(_translate("MainWindow", "Mac&hine Type"))
        self.menu_Measurement_units.setTitle(_translate("MainWindow", "Measurement &units"))
        self.menuHelp.setTitle(_translate("MainWindow", "&Help"))
        self.actionOpen.setText(_translate("MainWindow", "&Open..."))
        self.actionOpen.setStatusTip(_translate("MainWindow", "Load DXF or other supported document"))
        self.actionOpen.setShortcut(_translate("MainWindow", "Ctrl+O"))
        self.actionClose.setText(_translate("MainWindow", "&Exit"))
        self.actionClose.setStatusTip(_translate("MainWindow", "Exit DXF2GCODE and close window"))
        self.actionClose.setShortcut(_translate("MainWindow", "Ctrl+Q"))
        self.actionShowPathDirections.setText(_translate("MainWindow", "&Show Path Directions"))
        self.actionShowPathDirections.setStatusTip(_translate("MainWindow", "Always shows the path direction in the plot (not only while selected)"))
        self.actionShowDisabledPaths.setText(_translate("MainWindow", "Show Disabled &Paths"))
        self.actionAutoscale.setText(_translate("MainWindow", "&Autoscale"))
        self.actionAutoscale.setShortcut(_translate("MainWindow", "Ctrl+V, A"))
        self.actionDeleteG0Paths.setText(_translate("MainWindow", "&Delete G0 Paths"))
        self.actionConfiguration.setText(_translate("MainWindow", "&Configuration..."))
        self.actionConfiguration.setShortcut(_translate("MainWindow", "Ctrl+Shift+C"))
        self.actionConfigurationPostprocessor.setText(_translate("MainWindow", "&Postprocessor configuration..."))
        self.actionConfigurationPostprocessor.setShortcut(_translate("MainWindow", "Ctrl+Shift+P"))
        self.actionTolerances.setText(_translate("MainWindow", "&Tolerances"))
        self.actionScaleAll.setText(_translate("MainWindow", "&Scale All"))
        self.actionRotateAll.setText(_translate("MainWindow", "&Rotate All"))
        self.actionMoveWorkpieceZero.setText(_translate("MainWindow", "&Move Workpiece Zero"))
        self.actionAbout.setText(_translate("MainWindow", "&About"))
        self.actionOptimizePaths.setText(_translate("MainWindow", "&Optimize Paths"))
        self.actionOptimizePaths.setShortcut(_translate("MainWindow", "Ctrl+Shift+O"))
        self.actionExportShapes.setText(_translate("MainWindow", "&Export Shapes"))
        self.actionExportShapes.setShortcut(_translate("MainWindow", "Ctrl+Shift+E"))
        self.actionOptimizeAndExportShapes.setText(_translate("MainWindow", "Optimize &and Export Shapes"))
        self.actionOptimizeAndExportShapes.setShortcut(_translate("MainWindow", "Ctrl+E"))
        self.actionLiveUpdateExportRoute.setText(_translate("MainWindow", "&Live Update Export Route"))
        self.actionReload.setText(_translate("MainWindow", "&Reload"))
        self.actionReload.setShortcut(_translate("MainWindow", "Ctrl+R"))
        self.actionSplitLineSegments.setText(_translate("MainWindow", "Split &Line Segments"))
        self.actionSplitLineSegments.setStatusTip(_translate("MainWindow", "Split line segments, e.g. can be used for compensation (G41/G42) in combination with EMC"))
        self.actionAutomaticCutterCompensation.setText(_translate("MainWindow", "&Automatic Cutter Compensation"))
        self.actionMilling.setText(_translate("MainWindow", "&Milling"))
        self.actionDragKnife.setText(_translate("MainWindow", "&Drag Knife"))
        self.actionLathe.setText(_translate("MainWindow", "&Lathe"))
        self.actionTopView.setText(_translate("MainWindow", "&Top View"))
        self.actionTopView.setShortcut(_translate("MainWindow", "Ctrl+V, T"))
        self.actionIsometricView.setText(_translate("MainWindow", "&Isometric View"))
        self.actionIsometricView.setShortcut(_translate("MainWindow", "Ctrl+V, I"))
        self.actionSaveProjectAs.setText(_translate("MainWindow", "&Save Project As..."))
        self.actionSaveProjectAs.setShortcut(_translate("MainWindow", "Ctrl+S"))
        self.actionMillimeters.setText(_translate("MainWindow", "&Millimeters"))
        self.actionInches.setText(_translate("MainWindow", "&Inches"))
from dxf2gcode.gui.canvas import Canvas
from dxf2gcode.gui.messagebox import MessageBox
from dxf2gcode.gui.treeview import TreeView
import dxf2gcode_images5_rc
